/****************************************************************************
**
** This file is part of a Qt Solutions component.
** 
** Copyright (c) 2009 Nokia Corporation and/or its subsidiary(-ies).
** 
** Contact:  Qt Software Information (qt-info@nokia.com)
** 
** Commercial Usage  
** Licensees holding valid Qt Commercial licenses may use this file in
** accordance with the Qt Solutions Commercial License Agreement provided
** with the Software or, alternatively, in accordance with the terms
** contained in a written agreement between you and Nokia.
** 
** GNU Lesser General Public License Usage
** Alternatively, this file may be used under the terms of the GNU Lesser
** General Public License version 2.1 as published by the Free Software
** Foundation and appearing in the file LICENSE.LGPL included in the
** packaging of this file.  Please review the following information to
** ensure the GNU Lesser General Public License version 2.1 requirements
** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
** 
** In addition, as a special exception, Nokia gives you certain
** additional rights. These rights are described in the Nokia Qt LGPL
** Exception version 1.0, included in the file LGPL_EXCEPTION.txt in this
** package.
** 
** GNU General Public License Usage 
** Alternatively, this file may be used under the terms of the GNU
** General Public License version 3.0 as published by the Free Software
** Foundation and appearing in the file LICENSE.GPL included in the
** packaging of this file.  Please review the following information to
** ensure the GNU General Public License version 3.0 requirements will be
** met: http://www.gnu.org/copyleft/gpl.html.
** 
** Please note Third Party Software included with Qt Solutions may impose
** additional restrictions and it is the user's responsibility to ensure
** that they have met the licensing requirements of the GPL, LGPL, or Qt
** Solutions Commercial license and the relevant license of the Third
** Party Software they are using.
** 
** If you are unsure which license is appropriate for your use, please
** contact the sales department at qt-sales@nokia.com.
** 
****************************************************************************/



#include <QtGui>
#if defined(QT_EXPERIMENTAL_SOLUTION)
# include "qstate.h"
# include "qstatemachine.h"
# include "qabstractstategroup.h"
# include "qtransition.h"
# include "qparallelanimationgroup.h"
# include "qsequentialanimationgroup.h"
# include "qpropertyanimation.h"
#endif

class GraphicsWidget : public QGraphicsWidget
{
    Q_OBJECT
    Q_PROPERTY(QRectF geometry READ geometry WRITE setGeometry)
};

class GraphicsProxyWidget : public QGraphicsProxyWidget
{
    Q_OBJECT
    Q_PROPERTY(QRectF geometry READ geometry WRITE setGeometry)
};

class Pixmap : public GraphicsWidget
{
    Q_OBJECT
    Q_PROPERTY(qreal rotationZ READ rotationZ WRITE setRotationZ)
    Q_PROPERTY(qreal opacity READ opacity WRITE setOpacity)
public:
    Pixmap(const QPixmap &pix)
        : GraphicsWidget(), p(pix), currentRotation(0), opac(1)
    {
        setSizePolicy(QSizePolicy::Fixed, QSizePolicy::Fixed);
    }

    void setOpacity(qreal opacity)
    {
        opac = opacity;
        update();
    }

    qreal opacity() const
    {
        return opac;
    }

    void paint(QPainter *painter, const QStyleOptionGraphicsItem *, QWidget *)
    {
        painter->setOpacity(opac);
        painter->drawPixmap(QPointF(), p);
    }

    qreal rotationZ() const
    {
        return currentRotation;
    }

    void setRotationZ(qreal rr)
    {
        currentRotation = rr;
        QPointF c = boundingRect().center();
        setTransform(QTransform().translate(c.x(), c.y()).rotate(rr, Qt::ZAxis).translate(-c.x(), -c.y()));
    }

protected:
    QSizeF sizeHint(Qt::SizeHint, const QSizeF & = QSizeF())
    {
        return QSizeF(p.width(), p.height());
    }

private:
    QPixmap p;
    qreal currentRotation;
    qreal opac;
};

int main(int argc, char *argv[])
{
    Q_INIT_RESOURCE(states);

    QApplication app(argc, argv);

    // Text edit and button
    QTextEdit *edit = new QTextEdit;
    edit->setText("asdf lkjha yuoiqwe asd iuaysd u iasyd uiy "
                  "asdf lkjha yuoiqwe asd iuaysd u iasyd uiy "
                  "asdf lkjha yuoiqwe asd iuaysd u iasyd uiy "
                  "asdf lkjha yuoiqwe asd iuaysd u iasyd uiy!");

    QPushButton *button = new QPushButton;
    QGraphicsProxyWidget *buttonProxy = new GraphicsProxyWidget;
    buttonProxy->setWidget(button);
    QGraphicsProxyWidget *editProxy = new GraphicsProxyWidget;
    editProxy->setWidget(edit);

    QGroupBox *box = new QGroupBox;
    box->setFlat(true);
    box->setTitle("Options");

    QVBoxLayout *layout2 = new QVBoxLayout;
    box->setLayout(layout2);
    layout2->addWidget(new QRadioButton("Herring"));
    layout2->addWidget(new QRadioButton("Blue Parrot"));
    layout2->addWidget(new QRadioButton("Petunias"));
    layout2->addStretch();

    QGraphicsProxyWidget *boxProxy = new GraphicsProxyWidget;
    boxProxy->setWidget(box);

    // Parent widget
    QGraphicsWidget *widget = new GraphicsWidget;
    QGraphicsLinearLayout *layout = new QGraphicsLinearLayout(Qt::Vertical, widget);
    layout->addItem(editProxy);
    layout->addItem(buttonProxy);
    widget->setLayout(layout);

    Pixmap *p1 = new Pixmap(QPixmap(":/digikam.png"));
    Pixmap *p2 = new Pixmap(QPixmap(":/akregator.png"));
    Pixmap *p3 = new Pixmap(QPixmap(":/accessories-dictionary.png"));
    Pixmap *p4 = new Pixmap(QPixmap(":/k3b.png"));
    Pixmap *p5 = new Pixmap(QPixmap(":/help-browser.png"));
    Pixmap *p6 = new Pixmap(QPixmap(":/kchart.png"));

    QGraphicsScene scene(0, 0, 400, 300);
    scene.setBackgroundBrush(scene.palette().window());
    scene.addItem(widget);
    scene.addItem(boxProxy);
    scene.addItem(p1);
    scene.addItem(p2);
    scene.addItem(p3);
    scene.addItem(p4);
    scene.addItem(p5);
    scene.addItem(p6);

    QtStateMachine machine;
    QtAbstractStateGroup *root = machine.rootState();
    QtState *state1 = new QtState(root);
    QtState *state2 = new QtState(root);
    QtState *state3 = new QtState(root);
    machine.setInitialState(state1);

    // State 1
    state1->setPropertyOnEntry(button, "text", "Switch to state 2");
    state1->setPropertyOnEntry(widget, "geometry", QRectF(0, 0, 400, 150));
    state1->setPropertyOnEntry(box, "geometry", QRect(-200, 150, 200, 150));
    state1->setPropertyOnEntry(p1, "geometry", QRectF(68, 185, 64, 64));
    state1->setPropertyOnEntry(p2, "geometry", QRectF(168, 185, 64, 64));
    state1->setPropertyOnEntry(p3, "geometry", QRectF(268, 185, 64, 64));
    state1->setPropertyOnEntry(p4, "geometry", QRectF(68-150, 48-150, 64, 64));
    state1->setPropertyOnEntry(p5, "geometry", QRectF(168, 48-150, 64, 64));
    state1->setPropertyOnEntry(p6, "geometry", QRectF(268+150, 48-150, 64, 64));
    state1->setPropertyOnEntry(p1, "rotationZ", qreal(0));
    state1->setPropertyOnEntry(p2, "rotationZ", qreal(0));
    state1->setPropertyOnEntry(p3, "rotationZ", qreal(0));
    state1->setPropertyOnEntry(p4, "rotationZ", qreal(-270));
    state1->setPropertyOnEntry(p5, "rotationZ", qreal(-90));
    state1->setPropertyOnEntry(p6, "rotationZ", qreal(270));
    state1->setPropertyOnEntry(boxProxy, "opacity", qreal(0));
    state1->setPropertyOnEntry(p1, "opacity", qreal(1));
    state1->setPropertyOnEntry(p2, "opacity", qreal(1));
    state1->setPropertyOnEntry(p3, "opacity", qreal(1));
    state1->setPropertyOnEntry(p4, "opacity", qreal(0));
    state1->setPropertyOnEntry(p5, "opacity", qreal(0));
    state1->setPropertyOnEntry(p6, "opacity", qreal(0));

    // State 2
    state2->setPropertyOnEntry(button, "text", "Switch to state 3");
    state2->setPropertyOnEntry(widget, "geometry", QRectF(200, 150, 200, 150));
    state2->setPropertyOnEntry(box, "geometry", QRect(9, 150, 190, 150));
    state2->setPropertyOnEntry(p1, "geometry", QRectF(68-150, 185+150, 64, 64));
    state2->setPropertyOnEntry(p2, "geometry", QRectF(168, 185+150, 64, 64));
    state2->setPropertyOnEntry(p3, "geometry", QRectF(268+150, 185+150, 64, 64));
    state2->setPropertyOnEntry(p4, "geometry", QRectF(64, 48, 64, 64));
    state2->setPropertyOnEntry(p5, "geometry", QRectF(168, 48, 64, 64));
    state2->setPropertyOnEntry(p6, "geometry", QRectF(268, 48, 64, 64));
    state2->setPropertyOnEntry(p1, "rotationZ", qreal(-270));
    state2->setPropertyOnEntry(p2, "rotationZ", qreal(90));
    state2->setPropertyOnEntry(p3, "rotationZ", qreal(270));
    state2->setPropertyOnEntry(p4, "rotationZ", qreal(0));
    state2->setPropertyOnEntry(p5, "rotationZ", qreal(0));
    state2->setPropertyOnEntry(p6, "rotationZ", qreal(0));
    state2->setPropertyOnEntry(boxProxy, "opacity", qreal(1));
    state2->setPropertyOnEntry(p1, "opacity", qreal(0));
    state2->setPropertyOnEntry(p2, "opacity", qreal(0));
    state2->setPropertyOnEntry(p3, "opacity", qreal(0));
    state2->setPropertyOnEntry(p4, "opacity", qreal(1));
    state2->setPropertyOnEntry(p5, "opacity", qreal(1));
    state2->setPropertyOnEntry(p6, "opacity", qreal(1));

    // State 3
    state3->setPropertyOnEntry(button, "text", "Switch to state 1");
    state3->setPropertyOnEntry(p1, "geometry", QRectF(5, 5, 64, 64));
    state3->setPropertyOnEntry(p2, "geometry", QRectF(5, 5 + 64 + 5, 64, 64));
    state3->setPropertyOnEntry(p3, "geometry", QRectF(5, 5 + (64 + 5) + 64, 64, 64));
    state3->setPropertyOnEntry(p4, "geometry", QRectF(5 + 64 + 5, 5, 64, 64));
    state3->setPropertyOnEntry(p5, "geometry", QRectF(5 + 64 + 5, 5 + 64 + 5, 64, 64));
    state3->setPropertyOnEntry(p6, "geometry", QRectF(5 + 64 + 5, 5 + (64 + 5) + 64, 64, 64));
    state3->setPropertyOnEntry(widget, "geometry", QRectF(138, 5, 400 - 138, 200));
    state3->setPropertyOnEntry(box, "geometry", QRect(5, 205, 400, 90));
    state3->setPropertyOnEntry(p1, "opacity", qreal(1));
    state3->setPropertyOnEntry(p2, "opacity", qreal(1));
    state3->setPropertyOnEntry(p3, "opacity", qreal(1));
    state3->setPropertyOnEntry(p4, "opacity", qreal(1));
    state3->setPropertyOnEntry(p5, "opacity", qreal(1));
    state3->setPropertyOnEntry(p6, "opacity", qreal(1));

    QtParallelAnimationGroup animation1;

    QtSequentialAnimationGroup *animation1SubGroup;
    animation1SubGroup = new QtSequentialAnimationGroup(&animation1);
    animation1SubGroup->addPause(250);
    animation1SubGroup->add(new QtPropertyAnimation(box, "geometry"));

    animation1.add(new QtPropertyAnimation(widget, "geometry"));
    animation1.add(new QtPropertyAnimation(p1, "geometry"));
    animation1.add(new QtPropertyAnimation(p2, "geometry"));
    animation1.add(new QtPropertyAnimation(p3, "geometry"));
    animation1.add(new QtPropertyAnimation(p4, "geometry"));
    animation1.add(new QtPropertyAnimation(p5, "geometry"));
    animation1.add(new QtPropertyAnimation(p6, "geometry"));
    animation1.add(new QtPropertyAnimation(p1, "rotationZ"));
    animation1.add(new QtPropertyAnimation(p2, "rotationZ"));
    animation1.add(new QtPropertyAnimation(p3, "rotationZ"));
    animation1.add(new QtPropertyAnimation(p4, "rotationZ"));
    animation1.add(new QtPropertyAnimation(p5, "rotationZ"));
    animation1.add(new QtPropertyAnimation(p6, "rotationZ"));
    animation1.add(new QtPropertyAnimation(p1, "opacity"));
    animation1.add(new QtPropertyAnimation(p2, "opacity"));
    animation1.add(new QtPropertyAnimation(p3, "opacity"));
    animation1.add(new QtPropertyAnimation(p4, "opacity"));
    animation1.add(new QtPropertyAnimation(p5, "opacity"));
    animation1.add(new QtPropertyAnimation(p6, "opacity"));

    QtParallelAnimationGroup animation2;
    animation2.add(new QtPropertyAnimation(box, "geometry"));
    animation2.add(new QtPropertyAnimation(widget, "geometry"));
    animation2.add(new QtPropertyAnimation(p1, "geometry"));
    animation2.add(new QtPropertyAnimation(p2, "geometry"));
    animation2.add(new QtPropertyAnimation(p3, "geometry"));
    animation2.add(new QtPropertyAnimation(p4, "geometry"));
    animation2.add(new QtPropertyAnimation(p5, "geometry"));
    animation2.add(new QtPropertyAnimation(p6, "geometry"));
    animation2.add(new QtPropertyAnimation(p1, "rotationZ"));
    animation2.add(new QtPropertyAnimation(p2, "rotationZ"));
    animation2.add(new QtPropertyAnimation(p3, "rotationZ"));
    animation2.add(new QtPropertyAnimation(p4, "rotationZ"));
    animation2.add(new QtPropertyAnimation(p5, "rotationZ"));
    animation2.add(new QtPropertyAnimation(p6, "rotationZ"));
    animation2.add(new QtPropertyAnimation(p1, "opacity"));
    animation2.add(new QtPropertyAnimation(p2, "opacity"));
    animation2.add(new QtPropertyAnimation(p3, "opacity"));
    animation2.add(new QtPropertyAnimation(p4, "opacity"));
    animation2.add(new QtPropertyAnimation(p5, "opacity"));
    animation2.add(new QtPropertyAnimation(p6, "opacity"));

    QtParallelAnimationGroup animation3;
    animation3.add(new QtPropertyAnimation(box, "geometry"));
    animation3.add(new QtPropertyAnimation(widget, "geometry"));
    animation3.add(new QtPropertyAnimation(p1, "geometry"));
    animation3.add(new QtPropertyAnimation(p2, "geometry"));
    animation3.add(new QtPropertyAnimation(p3, "geometry"));
    animation3.add(new QtPropertyAnimation(p4, "geometry"));
    animation3.add(new QtPropertyAnimation(p5, "geometry"));
    animation3.add(new QtPropertyAnimation(p6, "geometry"));
    animation3.add(new QtPropertyAnimation(p1, "rotationZ"));
    animation3.add(new QtPropertyAnimation(p2, "rotationZ"));
    animation3.add(new QtPropertyAnimation(p3, "rotationZ"));
    animation3.add(new QtPropertyAnimation(p4, "rotationZ"));
    animation3.add(new QtPropertyAnimation(p5, "rotationZ"));
    animation3.add(new QtPropertyAnimation(p6, "rotationZ"));
    animation3.add(new QtPropertyAnimation(p1, "opacity"));
    animation3.add(new QtPropertyAnimation(p2, "opacity"));
    animation3.add(new QtPropertyAnimation(p3, "opacity"));
    animation3.add(new QtPropertyAnimation(p4, "opacity"));
    animation3.add(new QtPropertyAnimation(p5, "opacity"));
    animation3.add(new QtPropertyAnimation(p6, "opacity"));

    state1->addAnimatedTransition(button, SIGNAL(clicked()), state2, &animation1);
    state2->addAnimatedTransition(button, SIGNAL(clicked()), state3, &animation2);
    state3->addAnimatedTransition(button, SIGNAL(clicked()), state1, &animation3);

    machine.start();

    QGraphicsView view(&scene);
    view.show();

    return app.exec();
}

#include "main.moc"
