/****************************************************************************
**
** This file is part of a Qt Solutions component.
** 
** Copyright (c) 2009 Nokia Corporation and/or its subsidiary(-ies).
** 
** Contact:  Qt Software Information (qt-info@nokia.com)
** 
** Commercial Usage  
** Licensees holding valid Qt Commercial licenses may use this file in
** accordance with the Qt Solutions Commercial License Agreement provided
** with the Software or, alternatively, in accordance with the terms
** contained in a written agreement between you and Nokia.
** 
** GNU Lesser General Public License Usage
** Alternatively, this file may be used under the terms of the GNU Lesser
** General Public License version 2.1 as published by the Free Software
** Foundation and appearing in the file LICENSE.LGPL included in the
** packaging of this file.  Please review the following information to
** ensure the GNU Lesser General Public License version 2.1 requirements
** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
** 
** In addition, as a special exception, Nokia gives you certain
** additional rights. These rights are described in the Nokia Qt LGPL
** Exception version 1.0, included in the file LGPL_EXCEPTION.txt in this
** package.
** 
** GNU General Public License Usage 
** Alternatively, this file may be used under the terms of the GNU
** General Public License version 3.0 as published by the Free Software
** Foundation and appearing in the file LICENSE.GPL included in the
** packaging of this file.  Please review the following information to
** ensure the GNU General Public License version 3.0 requirements will be
** met: http://www.gnu.org/copyleft/gpl.html.
** 
** Please note Third Party Software included with Qt Solutions may impose
** additional restrictions and it is the user's responsibility to ensure
** that they have met the licensing requirements of the GPL, LGPL, or Qt
** Solutions Commercial license and the relevant license of the Third
** Party Software they are using.
** 
** If you are unsure which license is appropriate for your use, please
** contact the sales department at qt-sales@nokia.com.
** 
****************************************************************************/



#include "torpedo.h"
#include "pixmapitem.h"
#include "boat.h"
#include "graphicsscene.h"

#if defined(QT_EXPERIMENTAL_SOLUTION)
#include "qpropertyanimation.h"
#endif

Torpedo::Torpedo(QGraphicsItem * parent, Qt::WindowFlags wFlags)
    : QGraphicsWidget(parent,wFlags), currentSpeed(0), launchAnimation(0)
{
    pixmapItem = new PixmapItem(QString("torpedo"),GraphicsScene::Big, this);
    setZValue(2);
    setSizePolicy(QSizePolicy::Fixed, QSizePolicy::Fixed);
    setFlags(QGraphicsItem::ItemIsMovable);
    resize(pixmapItem->boundingRect().size());
}

void Torpedo::launch()
{
    launchAnimation = new QtPropertyAnimation(this, "pos");
    launchAnimation->setEndValue(QPointF(x(),qobject_cast<GraphicsScene *>(scene())->sealLevel() - 15));
    launchAnimation->setEasingCurve(QtEasingCurve::InQuad);
    launchAnimation->setDuration(y()/currentSpeed*10);
    launchAnimation->start();
    connect(launchAnimation,SIGNAL(finished()),this,SLOT(onAnimationLaunchFinished()));
    connect(launchAnimation,SIGNAL(valueChanged(const QVariant &)),this,SLOT(onAnimationLaunchValueChanged(const QVariant &)));
}

void Torpedo::setCurrentSpeed(int speed)
{
    if (speed < 0)
        return;
    currentSpeed = speed;
}

void Torpedo::onAnimationLaunchFinished()
{
    destroy();
}

void Torpedo::onAnimationLaunchValueChanged(const QVariant &)
{
    foreach (QGraphicsItem *item , collidingItems(Qt::IntersectsItemBoundingRect)) {
        if (item->type() == Boat::Type) {
            Boat *b = static_cast<Boat *>(item);
            b->destroy();
        }
    }
}

void Torpedo::destroy()
{
    currentSpeed = 0;
    if (launchAnimation) {
        disconnect(launchAnimation,SIGNAL(valueChanged(const QVariant &)),this,SLOT(onAnimationLaunchValueChanged(const QVariant &)));
        disconnect(launchAnimation,SIGNAL(finished()),this,SLOT(onAnimationLaunchFinished()));
        launchAnimation->stop();
        launchAnimation->deleteLater();
        launchAnimation = 0;
    }
    deleteLater();
}
